using System;
using System.Collections.Generic;
using System.IO;
using System.Linq;
using System.Runtime.InteropServices;
using System.Text;
using Autofac.Integration.Web;
using Autofac.Integration.Web.Forms;
using Microsoft.SharePoint;
using Microsoft.SharePoint.Administration;

namespace Autofac.Integration.SharePoint.Installer.Features.AutofacIntegration
{
    /// <summary>
    /// This class handles events raised during feature activation, deactivation, installation, uninstallation, and upgrade.
    /// </summary>
    /// <remarks>
    /// The GUID attached to this class may be used during packaging and should not be modified.
    /// </remarks>

    [Guid("9578f389-4e0b-40ef-9e8e-5358003e64d4")]
    public class AutofacIntegrationEventReceiver : SPFeatureReceiver
    {
        public override void FeatureActivated(SPFeatureReceiverProperties properties)
        {
            // generally equivalent to: SPFarm.Local.Services.GetValue<SPWebService>()
            var webService = properties.Feature.Parent as SPWebService;
            if (webService == null)
            {
                throw new ArgumentNullException("properties.Feature.Parent");
            }

            // let's register an autofac module (saves the config in the farm properties)
            SPContainerBuilder.RegisterModule(typeof(Playground.PlaygroundModule2).AssemblyQualifiedName, null, null, true);

            // let's make the necessary changes to register autofac in web.config and global.asax
            var globalAsaxFiles = new List<string>();
            var zones = Enum.GetValues(typeof (SPUrlZone)).Cast<SPUrlZone>().ToArray();
            foreach (var webApp in webService.WebApplications)
            {
                if (webApp.IsAdministrationWebApplication)
                    continue; // don't mess with central admin :-)

                // assuming every other web application "wants" to use Autofac
                // (this may not be the case, change logic here, or scope the feature differently)

                foreach (var entry in _autofacConfigEntries)
                {
                    webApp.WebConfigModifications.Add(entry.Prepare());
                }

                webApp.WebService.Update();
                webApp.WebService.ApplyWebConfigModifications();

                var paths =
                    zones.Select(z => Path.Combine(webApp.GetIisSettingsWithFallback(z).Path.ToString(), "global.asax"))
                        .Distinct().Where(File.Exists).ToArray();
                globalAsaxFiles.AddRange(paths);
            }

            // Now replace the global.asax file(s) - I dare you!! 
            try
            {
                var asaxFileContents =
                    string.Format(
                        @"<%@ Assembly Name=""Microsoft.SharePoint""%><%@ Assembly Name=""{0}""%><%@ Application Language=""C#"" Inherits=""{1}"" %>",
                        typeof(IContainerProviderAccessor).Assembly.FullName,
                        typeof(Autofac.Integration.SharePoint.GlobalApplication).AssemblyQualifiedName);
                foreach (var asax in globalAsaxFiles)
                {
                    File.WriteAllText(asax, asaxFileContents, Encoding.UTF8);
                }
            }
            catch
            {
                // Modify the global.asax files manually (this shouldn't happen)
            }
        }

        public override void FeatureDeactivating(SPFeatureReceiverProperties properties)
        {
            var webService = properties.Feature.Parent as SPWebService;
            if (webService == null)
            {
                // allow de-activation here
                return;
            }

            var globalAsaxFiles = new List<string>();
            var zones = Enum.GetValues(typeof (SPUrlZone)).Cast<SPUrlZone>().ToArray();
            foreach (var webApplication in webService.WebApplications)
            {
                var modsCollection = webApplication.WebConfigModifications;

                for (var i = modsCollection.Count - 1; i > -1; i--)
                {
                    if (modsCollection[i].Owner == ConfigModsOwnerName)
                    {
                        // Remove it and save the change to the configuration database  
                        modsCollection.Remove(modsCollection[i]);
                    }
                }
                webApplication.Update();

                // Reapply all the configuration modifications
                webApplication.WebService.Update();
                webApplication.WebService.ApplyWebConfigModifications();

                var paths =
                    zones.Select(z => Path.Combine(webApplication.GetIisSettingsWithFallback(z).Path.ToString(), "global.asax"))
                        .Distinct().Where(File.Exists).ToArray();
                globalAsaxFiles.AddRange(paths);
            }

            // Now replace the global.asax file(s)
            try
            {
                var asaxFileContents =
                    @"<%@ Assembly Name=""Microsoft.SharePoint""%><%@ Application Language=""C#"" Inherits=""Microsoft.SharePoint.ApplicationRuntime.SPHttpApplication"" %>";
                foreach (var asax in globalAsaxFiles)
                {
                    File.WriteAllText(asax, asaxFileContents, Encoding.UTF8);
                }
            }
            catch
            {
                // Modify the global.asax files manually (this shouldn't happen)
            }
        }

        #region Web.Config Modification Entries
        private const string ConfigModsOwnerName = "Autofac Integration For SharePoint";
        private readonly WebConfigEntry[] _autofacConfigEntries = 
            { 
                // configSections entry
                new WebConfigEntry( 
                    "section[@name='autofac']" 
                    ,"configuration/configSections" 
                    ,"<section name=\"autofac\" type=\"" + typeof(Autofac.Configuration.SectionHandler).AssemblyQualifiedName + "\"/>" 
                    ,SPWebConfigModification.SPWebConfigModificationType.EnsureChildNode 
                    ,false)  

                // autofac entry
                ,new WebConfigEntry(
                    "autofac" 
                    ,"configuration" 
                    ,"<autofac/>" 
                    ,SPWebConfigModification.SPWebConfigModificationType.EnsureChildNode 
                    ,false)   

                // autofac modules entry
                ,new WebConfigEntry(
                    "modules" 
                    ,"configuration/autofac" 
                    ,"<modules/>" 
                    ,SPWebConfigModification.SPWebConfigModificationType.EnsureChildNode 
                    ,false)
                // This is just a sample module
                ,new WebConfigEntry(
                    "module[@type='" + typeof(Playground.PlaygroundModule).AssemblyQualifiedName + "']"
                    ,"configuration/autofac/modules"
                    ,"<module type=\"" + typeof(Playground.PlaygroundModule).AssemblyQualifiedName + "\"/>"
                    ,SPWebConfigModification.SPWebConfigModificationType.EnsureChildNode 
                    , false)

                // autofac httpModules entries
                ,new WebConfigEntry( 
                    "add[@name='ContainerDisposal']" 
                    ,"configuration/system.web/httpModules" 
                    ,"<add name=\"ContainerDisposal\" type=\"" + typeof(ContainerDisposalModule).AssemblyQualifiedName + "\" />"
                    ,SPWebConfigModification.SPWebConfigModificationType.EnsureChildNode 
                    ,false) 
                ,new WebConfigEntry( 
                    "add[@name='PropertyInjection']" 
                    ,"configuration/system.web/httpModules" 
                    ,"<add name=\"PropertyInjection\" type=\"" + typeof(PropertyInjectionModule).AssemblyQualifiedName + "\" />"
                    ,SPWebConfigModification.SPWebConfigModificationType.EnsureChildNode 
                    ,false)
                ,new WebConfigEntry( 
                    "add[@name='AttributeInjection']" 
                    ,"configuration/system.web/httpModules" 
                    ,"<add name=\"AttributeInjection\" type=\"" + typeof(AttributedInjectionModule).AssemblyQualifiedName + "\" />"
                    ,SPWebConfigModification.SPWebConfigModificationType.EnsureChildNode 
                    ,false)
                    
                // autofac iis7 modules entries
                ,new WebConfigEntry( 
                    "add[@name='ContainerDisposal']" 
                    ,"configuration/system.webServer/modules" 
                    ,"<add name=\"ContainerDisposal\" type=\"" + typeof(ContainerDisposalModule).AssemblyQualifiedName + "\" preCondition=\"managedHandler\" />"
                    ,SPWebConfigModification.SPWebConfigModificationType.EnsureChildNode 
                    ,false) 
                ,new WebConfigEntry( 
                    "add[@name='PropertyInjection']" 
                    ,"configuration/system.webServer/modules" 
                    ,"<add name=\"PropertyInjection\" type=\"" + typeof(PropertyInjectionModule).AssemblyQualifiedName + "\" preCondition=\"managedHandler\" />"
                    ,SPWebConfigModification.SPWebConfigModificationType.EnsureChildNode 
                    ,false)
                ,new WebConfigEntry( 
                    "add[@name='AttributeInjection']" 
                    ,"configuration/system.webServer/modules" 
                    ,"<add name=\"AttributeInjection\" type=\"" + typeof(AttributedInjectionModule).AssemblyQualifiedName + "\" preCondition=\"managedHandler\" />"
                    ,SPWebConfigModification.SPWebConfigModificationType.EnsureChildNode 
                    ,false)

                // autofac assemblies entries
                ,new WebConfigEntry( 
                    "add[@assembly='" + typeof(Autofac.IContainer).Assembly.FullName + "']" 
                    ,"configuration/system.web/compilation/assemblies" 
                    ,"<add assembly=\"" + typeof(Autofac.IContainer).Assembly.FullName + "\" />"
                    ,SPWebConfigModification.SPWebConfigModificationType.EnsureChildNode 
                    ,false)
                ,new WebConfigEntry( 
                    "add[@assembly='" + typeof(Autofac.Configuration.ComponentElement).Assembly.FullName + "']" 
                    ,"configuration/system.web/compilation/assemblies" 
                    ,"<add assembly=\"" + typeof(Autofac.Configuration.ComponentElement).Assembly.FullName + "\" />"
                    ,SPWebConfigModification.SPWebConfigModificationType.EnsureChildNode 
                    ,false)
                ,new WebConfigEntry( 
                    "add[@assembly='" + typeof(IContainerProviderAccessor).Assembly.FullName + "']" 
                    ,"configuration/system.web/compilation/assemblies" 
                    ,"<add assembly=\"" + typeof(IContainerProviderAccessor).Assembly.FullName + "\" />"
                    ,SPWebConfigModification.SPWebConfigModificationType.EnsureChildNode 
                    ,false)
                    
                // add some safe control entries as well (assuming you'll be building some controls in Autofac.Integration.SharePoint)
                ,new WebConfigEntry( 
                    "SafeControl[@Assembly='" + typeof(Autofac.Integration.SharePoint.GlobalApplication).Assembly.FullName + "']"
                    ,"configuration/SharePoint/SafeControls" 
                    ,"<SafeControl Assembly=\"" + typeof(Autofac.Integration.SharePoint.GlobalApplication).Assembly.FullName + "\" Namespace=\"Autofac.Integration.SharePoint\" TypeName=\"*\" Safe=\"True\" SafeAgainstScript=\"False\" />"
                    ,SPWebConfigModification.SPWebConfigModificationType.EnsureChildNode 
                    ,false)

            };

        /// <summary> 
        /// Container to hold info about our modifications to the web.config. 
        /// </summary> 
        public class WebConfigEntry
        {
            public string Name;
            public string XPath;
            public string Value;
            public SPWebConfigModification.SPWebConfigModificationType ModificationType;
            public bool KeepOnDeactivate;

            public WebConfigEntry(string name, string xPath, string value,
                SPWebConfigModification.SPWebConfigModificationType modificationType, bool keepOnDeactivate)
            {
                Name = name;
                XPath = xPath;
                Value = value;
                ModificationType = modificationType;
                KeepOnDeactivate = keepOnDeactivate;
            }

            public SPWebConfigModification Prepare()
            {
                var modification = new SPWebConfigModification(Name, XPath)
                {
                    Owner = ConfigModsOwnerName,
                    Sequence = 0,
                    Type = ModificationType,
                    Value = Value
                };
                return modification;
            }
        }
        #endregion
    }
}
