﻿using System;
using System.Collections.Generic;
using System.ComponentModel;
using System.Configuration;
using System.Linq;
using System.Reflection;
using System.Xml.Linq;
using System.Xml.XPath;
using Autofac.Core;
using Autofac.Core.Activators.Reflection;
using Microsoft.SharePoint.Administration;
using Container = Autofac.Core.Container;

namespace Autofac.Integration.SharePoint
{
    public class SPContainerBuilder : ContainerBuilder
    {
        private const string AUTOFAC_MOD_KEY = "_autofac_modules";

        /// <summary>
        /// Intended only for SPFarm feature receiver
        /// </summary>
        public static void RemoveModule(string moduleType)
        {
            XElement xm = GetPersistedModules();
            if (xm.HasElements)
            {
                XElement[] matchingModules = xm.XPathSelectElements("module[@type='" + moduleType + "']").ToArray();
                if (matchingModules.Any())
                {
                    Console.WriteLine("Module Removed");
                    matchingModules.Remove();

                    UpdatePersistedModules(xm);
                }
            }
        }

        /// <summary>
        /// Intended only for SPFarm feature receiver
        /// </summary>
        public static void RegisterModule(string moduleType, IDictionary<string, string> properties,
                                          IDictionary<string, string> parameters, bool replaceIfExists)
        {
            // create the module
            var xe = new XElement(XName.Get("module", ""));
            xe.SetAttributeValue(XName.Get("type", ""), moduleType);
            if (parameters != null && parameters.Any())
            {
                var xepc = new XElement(XName.Get("parameters", ""));
                foreach (string parameter in parameters.Keys)
                {
                    var xep = new XElement(XName.Get("parameter", ""));
                    xep.SetAttributeValue(XName.Get("name", ""), parameter);
                    xep.SetAttributeValue(XName.Get("value", ""), parameters[parameter]);
                    xepc.Add(xep);
                }
                xe.Add(xepc);
            }
            if (properties != null && properties.Any())
            {
                var xepc = new XElement(XName.Get("properties", ""));
                foreach (string property in properties.Keys)
                {
                    var xep = new XElement(XName.Get("property", ""));
                    xep.SetAttributeValue(XName.Get("name", ""), property);
                    xep.SetAttributeValue(XName.Get("value", ""), properties[property]);
                    xepc.Add(xep);
                }
                xe.Add(xepc);
            }

            XElement xm = GetPersistedModules();
            bool exists = xm.XPathSelectElements("module[@type='" + moduleType + "']").Any();
            if (exists && !replaceIfExists)
                return;
            if (exists)
            {
                RemoveModule(moduleType);
                // refresh xm
                xm = GetPersistedModules();
            }
            xm.Add(xe);
            UpdatePersistedModules(xm);
        }

        private static void UpdatePersistedModules(XElement modules)
        {
            if (modules == null)
                return;

            SPFarm farm = SPFarm.Local;
            if (farm != null)
            {
                farm.Properties[AUTOFAC_MOD_KEY] = modules.ToString();
                farm.Update();
            }
        }

        private static XElement GetPersistedModules()
        {
            SPFarm farm = SPFarm.Local;
            if (farm != null && farm.Properties.ContainsKey(AUTOFAC_MOD_KEY))
            {
                string moduleXml = farm.Properties[AUTOFAC_MOD_KEY].ToString();
                if (!string.IsNullOrEmpty(moduleXml))
                    return XElement.Parse(moduleXml);
            }
            return XElement.Parse("<modules/>");
        }

        #region Persisted Module to IModule Conversion

        private static IEnumerable<Parameter> ToParameters(IEnumerable<XElement> elements)
        {
            var ps = new List<Parameter>();
            foreach (XElement element in elements)
            {
                var p =
                    (Parameter)
                    new ResolvedParameter(((pi, c) => pi.Name == element.Attribute(XName.Get("name", "")).Value),
                                          ((pi, c) =>
                                           AlternateTypeManipulation.ChangeToCompatibleType(
                                               element.Attribute(XName.Get("value", "")).Value, pi.ParameterType)));
            }
            return ps;
        }

        internal static IList<IModule> GetFarmPropertyConfiguredModules()
        {
            XElement xModules = GetPersistedModules();

            var iModules = new List<IModule>();
            foreach (XElement xModule in xModules.XPathSelectElements("module"))
            {
                if (xModule.Attribute(XName.Get("type", "")) == null)
                    continue;

                string typeName = xModule.Attribute(XName.Get("type", "")).Value;
                IEnumerable<XElement> xParameters = xModule.XPathSelectElements("parameters/parameter");
                IEnumerable<XElement> xProperties = xModule.XPathSelectElements("properties/property");
                IEnumerable<Parameter> parameters = ToParameters(xParameters);
                IEnumerable<Parameter> properties = ToParameters(xProperties);

                Type type = Type.GetType(typeName);
                if (type != null)
                {
                    var module =
                        (IModule)
                        new ReflectionActivator(type, new BindingFlagsConstructorFinder(BindingFlags.Public),
                                                new MostParametersConstructorSelector(), parameters, properties).
                            ActivateInstance(Container.Empty, Enumerable.Empty<Parameter>());
                    iModules.Add(module);
                }
            }
            return iModules;
        }

        internal class AlternateTypeManipulation
        {
            public static object ChangeToCompatibleType(object value, Type destinationType)
            {
                if (destinationType == null)
                    throw new ArgumentNullException("destinationType");
                if (value == null)
                {
                    if (destinationType.IsValueType)
                        return Activator.CreateInstance(destinationType);
                    else
                        return null;
                }
                else
                {
                    TypeConverter converter1 = TypeDescriptor.GetConverter(value.GetType());
                    if (converter1 != null && converter1.CanConvertTo(destinationType))
                        return converter1.ConvertTo(value, destinationType);
                    if (destinationType.IsAssignableFrom(value.GetType()))
                        return value;
                    TypeConverter converter2 = TypeDescriptor.GetConverter(destinationType);
                    if (converter2 == null)
                        throw new ConfigurationErrorsException(string.Format("Cannot convert type from {0} to {1}.",
                                                                             (object) value.GetType(),
                                                                             (object) destinationType));
                    else
                        return converter2.ConvertFrom(value);
                }
            }
        }

        #endregion
    }
}