﻿using System;
using System.Collections.Generic;
using System.IO;
using Autofac.Configuration;
using Autofac.Core;
using Microsoft.SharePoint.Administration;

namespace Autofac.Integration.SharePoint
{
    public class SPServiceLocator
    {
        private static SPContainerProvider _containerProvider;
        private static bool _initialized;

        internal static SPContainerProvider ContainerProvider
        {
            get { return _containerProvider; }
        }

        public static bool RequestLifetimeIsUnavailable
        {
            get { return _containerProvider.RequestLifetimeIsUnavailable; }
        }

        internal static void InitializeContainer()
        {
            InitializeContainer((string) null);
        }

        /// <summary>
        /// This method is available for testing, or to use the ServiceLocator capability outside of
        /// the SharePoint HttpContext (i.e.: console app, linqpad, powershell, etc...)
        /// <remarks>Client MUST be on the SharePoint server</remarks>
        /// </summary>
        /// <param name="webApplication"></param>
        public static void InitializeContainer(SPWebApplication webApplication)
        {
            InitializeContainer(webApplication, null);
        }

        public static void InitializeContainer(SPWebApplication webApplication, Action<ContainerBuilder> additionalBuild)
        {
            string configFile =
                Path.Combine(webApplication.GetIisSettingsWithFallback(SPUrlZone.Default).Path.ToString(), "web.config");
            InitializeContainer(configFile, additionalBuild);
        }

        public static void InitializeContainer(string configFile)
        {
            InitializeContainer(configFile, null);
        }

        /// <summary>
        /// This method is available for testing, or to use the ServiceLocator capability outside of
        /// the SharePoint HttpContext (i.e.: console app, linqpad, powershell, etc...)
        /// </summary>
        /// <param name="configFile">Autofac configuration file</param>
        /// <param name="additionalBuild">Any additional build information desired (will override the web.config and farm properties)</param>
        public static void InitializeContainer(string configFile, Action<ContainerBuilder> additionalBuild)
        {
            if (_initialized)
                return;

            if (!string.IsNullOrEmpty(configFile) && !File.Exists(configFile))
                throw new FileNotFoundException("The autofac configuration file could not be found.", configFile);

            var builder = new SPContainerBuilder();

            // register modules from config file
            builder.RegisterModule(string.IsNullOrEmpty(configFile)
                                       ? new ConfigurationSettingsReader("autofac")
                                       : new ConfigurationSettingsReader("autofac", configFile));

            // register modules from SPFarm.Properties
            IList<IModule> modules = SPContainerBuilder.GetFarmPropertyConfiguredModules();
            foreach (IModule module in modules)
                builder.RegisterModule(module);

            if (additionalBuild != null)
                additionalBuild.Invoke(builder);

            _containerProvider = new SPContainerProvider(builder.Build());
            _initialized = true;
        }

        public static ILifetimeScope NewDisposableLifetime()
        {
            return _containerProvider.NewDisposableLifetime();
        }

        public static ILifetimeScope NewDisposableLifetime(string name)
        {
            return _containerProvider.NewDisposableLifetime(name);
        }

        public static ILifetimeScope GetRequestLifetime()
        {
            return _containerProvider.GetRequestLifetime();
        }

        public static void Kill()
        {
            if (_containerProvider != null && _containerProvider.ApplicationContainer != null)
            {
                _containerProvider.ApplicationContainer.Dispose();
                _containerProvider = null;
                _initialized = false;
            }
        }
    }
}